@@ -1,5 +1,5 @@
 ### Sunthud Pornprasertmanit, Terrence D. Jorgensen, Yves Rosseel
-### Last updated: 27 May 2020
+### Last updated: 10 January 2021
 
 
 ## -------------
@@ -44,7 +44,7 @@
 ##' second coefficients omega will have the same value when the model has simple
 ##' structure, but different values when there are (for example) cross-loadings
 ##' or method factors. The first coefficient omega can be viewed as the
-##' reliability controlling for the other factors (like \eqn{\eta^2_partial} in
+##' reliability controlling for the other factors (like \eqn{\eta^2_{partial}} in
 ##' ANOVA). The second coefficient omega can be viewed as the unconditional
 ##' reliability (like \eqn{\eta^2} in ANOVA).
 ##'
@@ -85,7 +85,8 @@
 ##'
 ##' Regarding categorical indicators, coefficient alpha and AVE are calculated
 ##' based on polychoric correlations. The coefficient alpha from this function
-##' may be not the same as the standard alpha calculation for categorical items.
+##' differs from the standard alpha calculation, which does not assume items are
+##' continuous, so numerically weighted categories can be treated as numeric.
 ##' Researchers may check the \code{alpha} function in the \code{psych} package
 ##' for the standard coefficient alpha calculation.
 ##'
@@ -151,26 +152,26 @@
 ##' @references
 ##' Bollen, K. A. (1980). Issues in the comparative measurement of
 ##' political democracy. \emph{American Sociological Review, 45}(3), 370--390.
-##' doi:10.2307/2095172
+##' \doi{10.2307/2095172}
 ##'
 ##' Bentler, P. M. (1972). A lower-bound method for the dimension-free
 ##' measurement of internal consistency. \emph{Social Science Research, 1}(4),
-##' 343--357. doi:10.1016/0049-089X(72)90082-8
+##' 343--357. \doi{10.1016/0049-089X(72)90082-8}
 ##'
 ##' Bentler, P. M. (2009). Alpha, dimension-free, and model-based internal
 ##' consistency reliability. \emph{Psychometrika, 74}(1), 137--143.
-##' doi:10.1007/s11336-008-9100-1
+##' \doi{10.1007/s11336-008-9100-1}
 ##'
 ##' Cronbach, L. J. (1951). Coefficient alpha and the internal structure of
-##' tests. \emph{Psychometrika, 16}(3), 297--334. doi:10.1007/BF02310555
+##' tests. \emph{Psychometrika, 16}(3), 297--334. \doi{10.1007/BF02310555}
 ##'
 ##' Fornell, C., & Larcker, D. F. (1981). Evaluating structural equation models
 ##' with unobservable variables and measurement errors. \emph{Journal of
-##' Marketing Research, 18}(1), 39--50. doi:10.2307/3151312
+##' Marketing Research, 18}(1), 39--50. \doi{10.2307/3151312}
 ##'
 ##' Green, S. B., & Yang, Y. (2009). Reliability of summed item scores using
 ##' structural equation modeling: An alternative to coefficient alpha.
-##' \emph{Psychometrika, 74}(1), 155--167. doi:10.1007/s11336-008-9099-3
+##' \emph{Psychometrika, 74}(1), 155--167. \doi{10.1007/s11336-008-9099-3}
 ##'
 ##' McDonald, R. P. (1999). \emph{Test theory: A unified treatment}. Mahwah, NJ:
 ##' Erlbaum.
@@ -178,12 +179,10 @@
 ##' Raykov, T. (2001). Estimation of congeneric scale reliability using
 ##' covariance structure analysis with nonlinear constraints \emph{British
 ##' Journal of Mathematical and Statistical Psychology, 54}(2), 315--323.
-##' doi:10.1348/000711001159582
+##' \doi{10.1348/000711001159582}
 ##'
 ##' @examples
 ##'
-##' library(lavaan)
-##'
 ##' HS.model <- ' visual  =~ x1 + x2 + x3
 ##'               textual =~ x4 + x5 + x6
 ##'               speed   =~ x7 + x8 + x9 '
@@ -276,6 +275,7 @@ reliability <- function(object, return.total = FALSE, dropSingle = TRUE,
 
 	anyCategorical <- lavInspect(object, "categorical")
 	threshold <- if (anyCategorical) getThreshold(object) else NULL
+	latScales <- if (anyCategorical) getScales(object) else NULL
 
 	result <- list()
 	warnHigher <- FALSE
@@ -325,14 +325,10 @@ reliability <- function(object, return.total = FALSE, dropSingle = TRUE,
 			  next
 			}
 
-			error[j] <- sum(te[[i]][index, index, drop = FALSE])
 			sigma <- S[[i]][index, index, drop = FALSE]
-			alpha[j] <- computeAlpha(sigma, length(index))
-			total[j] <- sum(sigma)
-			impliedTotal[j] <- sum(SigmaHat[[i]][index, index, drop = FALSE])
+			alpha[j] <- computeAlpha(sigma)
 			faccontrib <- ly[[i]][,j, drop = FALSE] %*% ve[[i]][j,j, drop = FALSE] %*% t(ly[[i]][,j, drop = FALSE])
 			truefac <- diag(faccontrib[index, index, drop = FALSE])
-			commonfac <- sum(faccontrib[index, index, drop = FALSE])
 			trueitem <- diag(truevar[index, index, drop = FALSE])
 			erritem <- diag(te[[i]][index, index, drop = FALSE])
 			if (sum(abs(trueitem - truefac)) < 0.00001) {
@@ -342,19 +338,24 @@ reliability <- function(object, return.total = FALSE, dropSingle = TRUE,
 			}
 			if (categorical) {
 				omega1[j] <- omegaCat(truevar = faccontrib[index, index, drop = FALSE],
-				                      implied = SigmaHat[[i]][index, index, drop = FALSE],
 				                      threshold = threshold[[i]][index],
+				                      scales = latScales[[i]][index],
 				                      denom = faccontrib[index, index, drop = FALSE] + te[[i]][index, index, drop = FALSE])
 				omega2[j] <- omegaCat(truevar = faccontrib[index, index, drop = FALSE],
-				                      implied = SigmaHat[[i]][index, index, drop = FALSE],
 				                      threshold = threshold[[i]][index],
+				                      scales = latScales[[i]][index],
 				                      denom = SigmaHat[[i]][index, index, drop = FALSE])
 				omega3[j] <- omegaCat(truevar = faccontrib[index, index, drop = FALSE],
-				                      implied = SigmaHat[[i]][index, index, drop = FALSE],
 				                      threshold = threshold[[i]][index],
+				                      scales = latScales[[i]][index],
 				                      denom = sigma)
 			} else {
-				omega1[j] <- commonfac / (commonfac + error[j])
+			  commonfac <- sum(faccontrib[index, index, drop = FALSE])
+			  error[j] <- sum(te[[i]][index, index, drop = FALSE])
+			  impliedTotal[j] <- sum(SigmaHat[[i]][index, index, drop = FALSE])
+			  total[j] <- sum(sigma)
+
+			  omega1[j] <- commonfac / (commonfac + error[j])
 				omega2[j] <- commonfac / impliedTotal[j]
 				omega3[j] <- commonfac / total[j]
 			}
@@ -362,20 +363,20 @@ reliability <- function(object, return.total = FALSE, dropSingle = TRUE,
 		}
 
 		if (return.total & length(facNames) > 1L) {
-		  alpha <- c(alpha, total = computeAlpha(S[[i]], nrow(S[[i]])))
+		  alpha <- c(alpha, total = computeAlpha(S[[i]]))
 		  #FIXME: necessary?    names(alpha) <- c(names(common), "total")
 		  if (categorical) {
 		    omega1 <- c(omega1, total = omegaCat(truevar = truevar,
-		                                         implied = SigmaHat[[i]],
 		                                         threshold = threshold[[i]],
+		                                         scales = latScales[[i]],
 		                                         denom = truevar + te[[i]]))
 		    omega2 <- c(omega2, total = omegaCat(truevar = truevar,
-		                                         implied = SigmaHat[[i]],
 		                                         threshold = threshold[[i]],
+		                                         scales = latScales[[i]],
 		                                         denom = SigmaHat[[i]]))
 		    omega3 <- c(omega3, total = omegaCat(truevar = truevar,
-		                                         implied = SigmaHat[[i]],
 		                                         threshold = threshold[[i]],
+		                                         scales = latScales[[i]],
 		                                         denom = S[[i]]))
 		  } else {
 		    omega1 <- c(omega1, total = sum(truevar) / (sum(truevar) + sum(te[[i]])))
@@ -526,12 +527,10 @@ reliability <- function(object, return.total = FALSE, dropSingle = TRUE,
 ##'
 ##' @examples
 ##'
-##' library(lavaan)
-##'
 ##' HS.model3 <- ' visual  =~ x1 + x2 + x3
 ##'                textual =~ x4 + x5 + x6
 ##'                speed   =~ x7 + x8 + x9
-##' 			          higher =~ visual + textual + speed'
+##' 			         higher =~ visual + textual + speed'
 ##'
 ##' fit6 <- cfa(HS.model3, data = HolzingerSwineford1939)
 ##' reliability(fit6) # Should provide a warning for the endogenous variables
@@ -765,7 +764,7 @@ reliabilityL2 <- function(object, secondFactor,
 ##'
 ##' @references
 ##' Li, H. (1997). A unifying expression for the maximal reliability of a linear
-##' composite. \emph{Psychometrika, 62}(2), 245--249. doi:10.1007/BF02295278
+##' composite. \emph{Psychometrika, 62}(2), 245--249. \doi{10.1007/BF02295278}
 ##'
 ##' Raykov, T. (2012). Scale construction and development using structural
 ##' equation modeling. In R. H. Hoyle (Ed.), \emph{Handbook of structural
@@ -891,27 +890,38 @@ maximalRelia <- function(object, omit.imps = c("no.conv","no.se")) {
 ## Hidden Functions
 ## ----------------
 
-computeAlpha <- function(S, k) k/(k - 1) * (1.0 - sum(diag(S)) / sum(S))
+computeAlpha <- function(S) {
+  k <- nrow(S)
+  k/(k - 1) * (1.0 - sum(diag(S)) / sum(S))
+}
 
 #' @importFrom stats cov2cor pnorm
-omegaCat <- function(truevar, implied, threshold, denom) {
-	# denom could be polychoric correlation, model-implied correlation, or model-implied without error correlation
-	polyc <- truevar
-	invstdvar <- 1 / sqrt(diag(implied))
-	polyr <- diag(invstdvar) %*% polyc %*% diag(invstdvar)
-	nitem <- ncol(implied)
+omegaCat <- function(truevar, threshold, scales, denom) {
+  ## must be in standardized latent scale
+  R <- diag(scales) %*% truevar %*% diag(scales)
+
+	## denom could be model-implied polychoric correlation assuming diagonal theta,
+	##       model-implied polychoric correlation accounting for error covariances,
+	##       or "observed" polychoric correlation matrix.
+  ## If parameterization="theta", standardize the polychoric coVARIANCE matrix
 	denom <- cov2cor(denom)
-	sumnum <- 0
-	addden <- 0
+
+	nitem <- ncol(denom)
+	## initialize sums of cumulative probabilities
+	sumnum <- 0 # numerator
+	addden <- 0 # denominator
+	## loop over all pairs of items
 	for (j in 1:nitem) {
   	for (jp in 1:nitem) {
+  	  ## initialize sums of cumulative probabilities *per item*
   		sumprobn2 <- 0
   		addprobn2 <- 0
-  		t1 <- threshold[[j]]
-  		t2 <- threshold[[jp]]
+  		## for each pair of items, loop over all their thresholds
+  		t1 <- threshold[[j]]  * scales[j] # on standardized latent scale
+  		t2 <- threshold[[jp]] * scales[jp]
   		for (c in 1:length(t1)) {
     		for (cp in 1:length(t2)) {
-    			sumprobn2 <- sumprobn2 + p2(t1[c], t2[cp], polyr[j, jp])
+    			sumprobn2 <- sumprobn2 + p2(t1[c], t2[cp], R[j, jp])
     			addprobn2 <- addprobn2 + p2(t1[c], t2[cp], denom[j, jp])
     		}
   		}
@@ -958,14 +968,13 @@ p2 <- function(t1, t2, r) {
 # }
 
 ##' @importFrom lavaan lavInspect lavNames
-##' @importFrom methods getMethod
 getThreshold <- function(object) {
 	ngroups <- lavInspect(object, "ngroups") #TODO: add nlevels when capable
 	ordnames <- lavNames(object, "ov.ord")
-	FITTED <- getMethod("fitted", class(object))(object)
+	EST <- lavInspect(object, "est")
 
 	if (ngroups == 1L) {
-	  thresholds <- FITTED$th
+	  thresholds <- EST$tau[,"threshold"]
 	  result <- lapply(ordnames,
 	                   function(nn) thresholds[grepl(nn, names(thresholds))])
 	  names(result) <- ordnames
@@ -973,7 +982,9 @@ getThreshold <- function(object) {
 	  result <- list(result)
 
 	} else {
-	  thresholds <- sapply(FITTED, "[[", i = "th", simplify = FALSE)
+	  allThr <- EST[which(names(EST) == "tau")]
+	  ## convert 1-column matrices to vectors, preserving rownames
+	  thresholds <- sapply(allThr, "[", j = "threshold", simplify = FALSE)
 	  result <- list()
 		group.label <- lavInspect(object, "group.label")
 
@@ -989,6 +1000,23 @@ getThreshold <- function(object) {
 	return(result)
 }
 
+##' @importFrom lavaan lavInspect lavNames
+getScales <- function(object) {
+  ngroups <- lavInspect(object, "ngroups") #TODO: add nlevels when capable
+  ordnames <- lavNames(object, "ov.ord") #TODO: use to allow mix of cat/con vars
+  EST <- lavInspect(object, "est")
+
+  if (ngroups == 1L) {
+    result <- list(EST$delta[,"scales"])
+  } else {
+    result <- lapply(EST[which(names(EST) == "delta")],
+                     function(x) x[,"scales"])
+    names(result) <- lavInspect(object, "group.label")
+  }
+
+  return(result)
+}
+
 invGeneralRelia <- function(w, truevar, totalvar) {
 	1 - (t(w) %*% truevar %*% w) / (t(w) %*% totalvar %*% w)
 }
